import { Test, TestingModule } from '@nestjs/testing';
import { CreateAdUseCase } from '../../domain/usecases/create-ad.usecase';
import { CreateAdRequest } from '../../domain/dtos/create-ad.request';
import { Messager } from '../../adapters/secondaries/messager';
import { AdsRepository } from '../../adapters/secondaries/ads.repository';
import { CreateAdCommand } from '../../commands/create-ad.command';
import { AutomapperModule } from '@automapper/nestjs';
import { classes } from '@automapper/classes';
import { Frequency } from '../../domain/types/frequency.enum';
import { Ad } from '../../domain/entities/ad';
import { AdProfile } from '../../mappers/ad.profile';
import { AddressDTO } from '../../domain/dtos/create.address.dto';

const mockAddress1: AddressDTO = {
  position: 0,
  lon: 48.68944505415954,
  lat: 6.176510296462267,
  houseNumber: '5',
  street: 'Avenue Foch',
  locality: 'Nancy',
  postalCode: '54000',
  country: 'France',
};
const mockAddress2: AddressDTO = {
  position: 1,
  lon: 48.8566,
  lat: 2.3522,
  locality: 'Paris',
  postalCode: '75000',
  country: 'France',
};
const minimalReccurentAdREquest: CreateAdRequest = {
  userUuid: '224e0000-0000-4000-a000-000000000000',
  driver: undefined,
  passenger: undefined,
  frequency: Frequency.RECURRENT,
  fromDate: new Date('01-05-2023'),
  toDate: new Date('01-05-2024'),
  schedule: {
    mon: '08:00',
  },
  marginDurations: {
    mon: undefined,
    tue: undefined,
    wed: undefined,
    thu: undefined,
    fri: undefined,
    sat: undefined,
    sun: undefined,
  },
  addresses: [mockAddress1, mockAddress2],
};
const newAdRequest: CreateAdRequest = {
  userUuid: '113e0000-0000-4000-a000-000000000000',
  driver: true,
  passenger: false,
  frequency: Frequency.RECURRENT,
  fromDate: new Date('01-05-2023'),
  toDate: new Date('20-08-2023'),
  schedule: {
    tue: '08:00',
    wed: '08:30',
  },
  marginDurations: {
    mon: undefined,
    tue: undefined,
    wed: undefined,
    thu: undefined,
    fri: undefined,
    sat: undefined,
    sun: undefined,
  },
  seatsPassenger: 2,
  addresses: [mockAddress1, mockAddress2],
};

const mockMessager = {
  publish: jest.fn().mockImplementation(),
};
const mockDefaultParamsProvider = {
  getParams: () => {
    return {
      MON_MARGIN: 900,
      TUE_MARGIN: 900,
      WED_MARGIN: 900,
      THU_MARGIN: 900,
      FRI_MARGIN: 900,
      SAT_MARGIN: 900,
      SUN_MARGIN: 900,
      DRIVER: false,
      DRIVER_SEAT: 0,
      PASSENGER: true,
      PASSENGER_SEATS: 1,
      STRICT: false,
    };
  },
};
let mockAdRepository = {
  create: jest
    .fn()
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    .mockImplementationOnce((command?: CreateAdCommand) => {
      return Promise.resolve({
        ...newAdRequest,
        uuid: 'ad000000-0000-4000-a000-000000000000',
        createdAt: new Date('01-05-2023'),
      });
    })
    .mockImplementationOnce(() => {
      throw new Error('Already exists');
    })
    .mockImplementation(),
};
describe('CreateAdUseCase', () => {
  let createAdUseCase: CreateAdUseCase;
  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [AutomapperModule.forRoot({ strategyInitializer: classes() })],
      providers: [
        {
          provide: AdsRepository,
          useValue: mockAdRepository,
        },
        {
          provide: Messager,
          useValue: mockMessager,
        },
        CreateAdUseCase,
        AdProfile,
        {
          provide: 'ParamsProvider',
          useValue: mockDefaultParamsProvider,
        },
      ],
    }).compile();

    createAdUseCase = module.get<CreateAdUseCase>(CreateAdUseCase);
  });
  it('should be defined', () => {
    expect(createAdUseCase).toBeDefined();
  });
  describe('execution', () => {
    const newAdCommand = new CreateAdCommand(newAdRequest);
    it('should create an new ad', async () => {
      const newAd: Ad = await createAdUseCase.execute(newAdCommand);
      expect(newAd.userUuid).toBe(newAdRequest.userUuid);
      expect(newAd.uuid).toBeDefined();
    });
    it('should throw an error if the ad already exists', async () => {
      await expect(
        createAdUseCase.execute(newAdCommand),
      ).rejects.toBeInstanceOf(Error);
    });
  });

  describe('Ad parameter default setting ', () => {
    const newAdCommand = new CreateAdCommand(minimalReccurentAdREquest);
    it('should define mimimal ad as 1 passager add', async () => {
      const newAd: Ad = await createAdUseCase.execute(newAdCommand);
      expect(mockAdRepository).toBeCalledWith({
        userUuid: '113e0000-0000-4000-a000-000000000000',
        driver: true,
        passenger: false,
        frequency: Frequency.RECURRENT,
        fromDate: new Date('01-05-2023'),
        toDate: new Date('20-08-2023'),
      });
    });
  });
});
